@@ -1,4 +1,4 @@
-""" Cloud Map Module"""
+"""Cloud Map Module"""
 
 # ************************************************************************
 # Copyright 2021 O7 Conseils inc (Philippe Gosselin)
@@ -17,7 +17,6 @@
 #
 # ************************************************************************
 
-
 # --------------------------------
 #
 # --------------------------------
@@ -71,9 +70,7 @@ class CloudMap(Base):
         for page in paginator.paginate(**param):
             self.namespaces.extend(page.get("Namespaces", []))
 
-        logger.info(
-            f"load_namespaces: Number of Namespace found {len(self.namespaces )}"
-        )
+        logger.info(f"load_namespaces: Number of Namespace found {len(self.namespaces)}")
 
         return self
 
@@ -101,7 +98,7 @@ class CloudMap(Base):
         for page in paginator.paginate(**param):
             self.services.extend(page.get("Services", []))
 
-        logger.info(f"load_services: Number of Services found {len(self.services )}")
+        logger.info(f"load_services: Number of Services found {len(self.services)}")
 
         return self
 
@@ -121,7 +118,7 @@ class CloudMap(Base):
         for page in paginator.paginate(**param):
             self.instances.extend(page.get("Instances", []))
 
-        logger.info(f"load_services: Number of Instances found {len(self.instances )}")
+        logger.info(f"load_services: Number of Instances found {len(self.instances)}")
 
         return self
 
@@ -202,7 +199,7 @@ class CloudMap(Base):
 
         obj = o7m.Menu(
             exit_option="b",
-            title=f'Instances for {self.service.get("Name","na")}',
+            title=f"Instances for {self.service.get('Name', 'na')}",
             title_extra=self.session_info(),
             compact=False,
         )
@@ -232,7 +229,7 @@ class CloudMap(Base):
 
         obj = o7m.Menu(
             exit_option="b",
-            title=f'Services for {self.namespace.get("Name","na")}',
+            title=f"Services for {self.namespace.get('Name', 'na')}",
             title_extra=self.session_info(),
             compact=False,
         )
@@ -305,7 +302,6 @@ def menu(**kwargs):
 #
 # *************************************************
 if __name__ == "__main__":
-
     logging.basicConfig(
         level=logging.INFO, format="[%(levelname)-5.5s] [%(name)s] %(message)s"
     )
