@@ -227,9 +227,9 @@ class Ecs(Base):
         # pprint.pprint(self.cluster)
         # pprint.pprint(self.cluster_instances)
 
-        print(f'Name: {self.cluster.get("clusterName", "")}')
-        print(f'Status: {self.cluster["status"]}')
-        print(f'Pending Tasks: {self.cluster["pendingTasksCount"]}')
+        print(f"Name: {self.cluster.get('clusterName', '')}")
+        print(f"Status: {self.cluster['status']}")
+        print(f"Pending Tasks: {self.cluster['pendingTasksCount']}")
 
         print("")
         print("List of Services")
@@ -332,19 +332,19 @@ class Ecs(Base):
         # pprint.pprint(self.task)
 
         print("")
-        print(f'Group: {self.task["group"]}')
-        print(f'Status: {self.task["lastStatus"]}')
-        print(f'Health: {self.task["healthStatus"]}')
-        print(f'Started At: {self.task["startedAt"]}')
+        print(f"Group: {self.task['group']}")
+        print(f"Status: {self.task['lastStatus']}")
+        print(f"Health: {self.task['healthStatus']}")
+        print(f"Started At: {self.task['startedAt']}")
 
         print("")
-        print(f'Launch Type: {self.task["launchType"]}')
-        print(f'CPU: {self.task["cpu"]}')
-        print(f'Memory: {self.task["memory"]}')
+        print(f"Launch Type: {self.task['launchType']}")
+        print(f"CPU: {self.task['cpu']}")
+        print(f"Memory: {self.task['memory']}")
 
         print("")
-        print(f'Availability Zone: {self.task["availabilityZone"]}')
-        print(f'Exec Command Enabled: {self.task["enableExecuteCommand"]}')
+        print(f"Availability Zone: {self.task['availabilityZone']}")
+        print(f"Exec Command Enabled: {self.task['enableExecuteCommand']}")
 
         print("List of Containers")
         Table(
@@ -377,7 +377,7 @@ class Ecs(Base):
         print("List of containers is task")
         containers = task.get("containers", [])
         for i, container in enumerate(containers):
-            print(f'{i} -> {container["name"]}')
+            print(f"{i} -> {container['name']}")
 
         key = o7i.input_int("Select container id : ")
         if key is None or key < 0 or key >= len(containers):
@@ -447,7 +447,7 @@ class Ecs(Base):
 
         obj = o7m.Menu(
             exit_option="b",
-            title=f'ECS Cluster - {self.cluster.get("clusterName","na")}',
+            title=f"ECS Cluster - {self.cluster.get('clusterName', 'na')}",
             title_extra=self.session_info(),
             compact=False,
         )
