@@ -20,6 +20,7 @@
 #
 # --------------------------------
 import ast
+import datetime
 import json
 import logging
 import os
@@ -56,6 +57,7 @@ class S3(Base):
         self.buckets: list = []
 
         self.bucket: str = ""
+        self.with_buckets_details: bool = False
         self.prefix: str = ""
         self.key: str = ""
 
@@ -120,12 +122,56 @@ class S3(Base):
         policy = {"Version": "2012-10-17", "Statement": policies}
 
         # https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/s3.html#S3.Client.put_bucket_policy
-        response = self.client.put_bucket_policy(
-            Bucket=bucket, Policy=json.dumps(policy)
+        response = self.client.put_bucket_policy(Bucket=bucket, Policy=json.dumps(policy))
+
+        return response
+
+    # *************************************************
+    #
+    # *************************************************
+    def set_basic_lifecycle_fule(self):
+        """Configure a basic lifecycle rule"""
+
+        bucket = self.bucket
+        rules = self.get_lifecycle_rules(bucket=bucket)
+        rules.append(
+            {
+                "ID": "RemovedNonCurrentAfter1Year",
+                "Status": "Enabled",
+                "Filter": {"Prefix": ""},
+                "NoncurrentVersionExpiration": {"NoncurrentDays": 365},
+                "AbortIncompleteMultipartUpload": {"DaysAfterInitiation": 14},
+            }
+        )
+        config = {"Rules": rules}
+
+        print(f"Config: {config}")
+
+        # https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/s3/client/put_bucket_lifecycle_configuration.html
+        response = self.client.put_bucket_lifecycle_configuration(
+            Bucket=bucket, LifecycleConfiguration=config
         )
 
         return response
 
+    # *************************************************
+    #
+    # *************************************************
+    def get_bucket_region(self, bucket: str):
+        """Get the region of a bucket"""
+
+        try:
+            response = self.client.get_bucket_location(Bucket=bucket)
+        except self.client.exceptions.NoSuchBucket:
+            return "Deleted"
+
+        region = response.get("LocationConstraint", "NA")
+
+        if region is None:
+            region = "us-east-1"
+
+        return region
+
     # *************************************************
     #
     # *************************************************
@@ -140,6 +186,169 @@ class S3(Base):
         policy = response.get("Policy", "")
         return ast.literal_eval(policy).get("Statement", [])
 
+    # *************************************************
+    #
+    # *************************************************
+    def get_lifecycle_rules(self, bucket: str):
+        """Load LifeCycle rules for a bucket"""
+
+        try:
+            response = self.client.get_bucket_lifecycle_configuration(Bucket=bucket)
+        except self.client.exceptions.from_code("NoSuchBucketPolicy"):
+            return []
+
+        return response.get("Rules", [])
+
+    # *************************************************
+    #
+    # *************************************************
+    def get_bucket_versioning(self, bucket: str):
+        """Load additional information for a bucket"""
+
+        try:
+            response = self.client.get_bucket_versioning(Bucket=bucket)
+        except self.client.exceptions.from_code("NoSuchBucketPolicy"):
+            return []
+
+        return {
+            "Status": response.get("Status", "Disabled"),
+            "MFADelete": response.get("MFADelete", "Disabled"),
+        }
+
+    # *************************************************
+    #
+    # *************************************************
+    def get_bucket_object_lock(self, bucket: str):
+        """Load additional information for a bucket"""
+
+        try:
+            response = self.client.get_object_lock_configuration(Bucket=bucket)
+        except self.client.exceptions.from_code("NoSuchBucketPolicy"):
+            return "Disabled"
+
+        return response.get("ObjectLockConfiguration", {}).get(
+            "ObjectLockEnabled", "Disabled"
+        )
+
+    # *************************************************
+    #
+    # *************************************************
+    def get_bucket_metrics(self, bucket: str = None):
+        """Get average CPU utilization for the last 5 minutes, 1 hour, and 1 day"""
+
+        if bucket is None:
+            bucket = self.bucket
+
+        cloudwatch = self.session.client("cloudwatch")
+        now = datetime.datetime.now(datetime.UTC)
+        periods = {
+            "1d": (now - datetime.timedelta(days=1), 86400),
+            "7d": (now - datetime.timedelta(days=7), 86400),
+            "14d": (now - datetime.timedelta(days=14), 86400),
+            "30d": (now - datetime.timedelta(days=30), 86400),
+            "60d": (now - datetime.timedelta(days=60), 86400),
+            "90d": (now - datetime.timedelta(days=90), 86400),
+        }
+
+        metrics = {
+            "StandardStorage": {
+                "dimensions": [
+                    {"Name": "BucketName", "Value": bucket},
+                    {"Name": "StorageType", "Value": "StandardStorage"},
+                ],
+                "statistic": "Average",
+                "metric": "BucketSizeBytes",
+            },
+            "DeepArchiveStorage": {
+                "dimensions": [
+                    {"Name": "BucketName", "Value": bucket},
+                    {"Name": "StorageType", "Value": "DeepArchiveStorage"},
+                ],
+                "statistic": "Average",
+                "metric": "BucketSizeBytes",
+            },
+            "DeepArchiveS3ObjectOverhead": {
+                "dimensions": [
+                    {"Name": "BucketName", "Value": bucket},
+                    {"Name": "StorageType", "Value": "DeepArchiveS3ObjectOverhead"},
+                ],
+                "statistic": "Average",
+                "metric": "BucketSizeBytes",
+            },
+        }
+
+        results = []
+        for metric, param in metrics.items():
+            result = {"Metric": metric, "Statistic": param["statistic"]}
+            for label, (start, period) in periods.items():
+                response = cloudwatch.get_metric_statistics(
+                    Namespace="AWS/S3",
+                    MetricName=param["metric"],
+                    Dimensions=param["dimensions"],
+                    StartTime=start,
+                    EndTime=now,
+                    Period=period,
+                    Statistics=[param["statistic"]],
+                )
+
+                # print(f"Response: {response}")
+                datapoints = response.get("Datapoints", [])
+                value = None
+                unit = "na"
+
+                if datapoints:
+                    # Get the latest datapoint
+                    datapoint = sorted(
+                        datapoints, key=lambda x: x["Timestamp"], reverse=False
+                    )[0]
+                    value = datapoint[param["statistic"]]
+                    unit = datapoint.get("Unit", "na")
+
+                # print(f"Average {metric} for {label}: {value} {unit}")
+                result[label] = value
+                result["Unit"] = unit
+
+            results.append(result)
+
+        Table(
+            TableParam(
+                title="Instance Statistics",
+                columns=[
+                    ColumnParam(title="Metric", type="str", data_col="Metric"),
+                    ColumnParam(title="Statistic", type="str", data_col="Statistic"),
+                    ColumnParam(title="90-Day", type="bytes", data_col="90d"),
+                    ColumnParam(title="60-Day", type="bytes", data_col="60d"),
+                    ColumnParam(title="30-Day", type="bytes", data_col="30d"),
+                    ColumnParam(title="14-Day", type="bytes", data_col="14d"),
+                    ColumnParam(title="7-Day", type="bytes", data_col="7d"),
+                    ColumnParam(title="1-Day", type="bytes", data_col="1d"),
+                    ColumnParam(title="Unit", type="txt", data_col="Unit"),
+                ],
+            ),
+            results,
+        ).print()
+        return results
+
+    # *************************************************
+    #
+    # *************************************************
+    def get_bucket_details(self, bucket) -> dict:
+        """get details for a bucket"""
+
+        ret = {}
+        ret["Region"] = self.get_bucket_region(bucket=bucket)
+        ret["Policies"] = self.get_bucket_policies(bucket=bucket)
+        ret["IsSSL"] = self.verify_ssl_requests_only(
+            bucket=bucket, statements=ret["Policies"]
+        )
+        ret["Versioning"] = self.get_bucket_versioning(bucket=bucket)
+        ret["VersioningStatus"] = ret["Versioning"].get("Status", "Disabled")
+        ret["LifecycleRules"] = self.get_lifecycle_rules(bucket=bucket)
+        ret["LifecycleRulesCount"] = len(ret["LifecycleRules"])
+        ret["ObjectLock"] = self.get_bucket_object_lock(bucket=bucket)
+
+        return ret
+
     # *************************************************
     #
     # *************************************************
@@ -155,25 +364,18 @@ class S3(Base):
         self.buckets = resp.get("Buckets", [])
         logger.info(f"LoadBuckets: Number of Bucket found: {len(self.buckets)}")
 
-        for bucket in self.buckets:
+        if self.with_buckets_details:
+            for bucket in self.buckets:
+                bucket.update(self.get_bucket_details(bucket=bucket["Name"]))
 
-            # Get region for each bucket
-            try:
-                resp = self.client.get_bucket_location(Bucket=bucket["Name"])
-            except self.client.exceptions.NoSuchBucket:
-                bucket["Region"] = "Deleted"
-                continue
-
-            bucket["Region"] = resp.get("LocationConstraint", "NA")
-            if bucket["Region"] is None:
-                bucket["Region"] = "us-east-1"
-
-            # Policies
-            bucket["Policies"] = self.get_bucket_policies(bucket=bucket["Name"])
-            bucket["IsSSL"] = self.verify_ssl_requests_only(
-                bucket=bucket["Name"], statements=bucket["Policies"]
-            )
+        return self
 
+    # *************************************************
+    #
+    # *************************************************
+    def load_buckets_extra(self):
+        """Load additional information for all buckets"""
+        self.with_buckets_details = True
         return self
 
     # *************************************************
@@ -184,13 +386,7 @@ class S3(Base):
 
         logger.info("load_bucket")
         bucket = self.bucket
-
-        self.bucket_info = {}
-        self.bucket_info["Policies"] = self.get_bucket_policies(bucket=bucket)
-        self.bucket_info["IsSSL"] = self.verify_ssl_requests_only(
-            bucket=bucket, statements=self.bucket_info["Policies"]
-        )
-
+        self.bucket_info = self.get_bucket_details(bucket=bucket)
         return self
 
     # *************************************************
@@ -231,7 +427,7 @@ class S3(Base):
 
         logger.info("load_folder")
 
-        # https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/s3/client/get_object.html
+        # https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/s3/client/get_object_attributes.html
         self.s3_object = self.client.get_object_attributes(
             Bucket=self.bucket,
             Key=self.key,
@@ -263,43 +459,6 @@ class S3(Base):
 
         print(f"Presigned URL: {url}")
 
-    # To Download file
-    # https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/s3/client/download_file.html
-
-    # Download directory, example from chat gpt:
-    # import os
-    # import boto3
-
-    # # Initialize a session using Amazon S3
-    # s3 = boto3.client('s3')
-
-    # # Create an S3 resource object
-    # s3_resource = boto3.resource('s3')
-
-    # # S3 bucket and folder details
-    # bucket_name = 'my-bucket'
-    # folder_name = 'my-folder/'
-
-    # # Local directory to save the files
-    # local_directory = './local-folder'
-
-    # # Make sure the local directory exists
-    # if not os.path.exists(local_directory):
-    #     os.makedirs(local_directory)
-
-    # # Initialize paginator and iterate through each page (helps with large number of files)
-    # paginator = s3.get_paginator('list_objects_v2')
-    # for page in paginator.paginate(Bucket=bucket_name, Prefix=folder_name):
-    #     # Download each file individually
-    #     for obj in page['Contents']:
-    #         key = obj['Key']
-    #         path, filename = os.path.split(key)
-    #         save_path = os.path.join(local_directory, filename)
-
-    #         # Download the file
-    #         print(f"Downloading {key} to {save_path}")
-    #         s3_resource.meta.client.download_file(bucket_name, key, save_path)
-
     # *************************************************
     #
     # *************************************************
@@ -357,10 +516,10 @@ class S3(Base):
         print(f"Prefix: {self.prefix}")
         print(f"Name  : {self.key}")
         print()
-        print(f'Last Modified: {self.s3_object.get("LastModified", "-")}')
-        print(f'Storage Class: {self.s3_object.get("StorageClass", "-")}')
-        print(f'ETag: {self.s3_object.get("ETag", "-")}')
-        print(f'Size: {o7f.to_bytes(self.s3_object.get("ObjectSize", "-"))}')
+        print(f"Last Modified: {self.s3_object.get('LastModified', '-')}")
+        print(f"Storage Class: {self.s3_object.get('StorageClass', '-')}")
+        print(f"ETag: {self.s3_object.get('ETag', '-')}")
+        print(f"Size: {o7f.to_bytes(self.s3_object.get('ObjectSize', '-'))}")
 
         print()
 
@@ -376,12 +535,36 @@ class S3(Base):
             columns=[
                 ColumnParam(title="id", type="i", min_width=4),
                 ColumnParam(title="Name", type="str", data_col="Name"),
-                ColumnParam(title="Created", type="datetime", data_col="CreationDate"),
+                ColumnParam(title="Created", type="date", data_col="CreationDate"),
                 ColumnParam(title="Region", type="str", data_col="Region"),
-                ColumnParam(title="Region", type="str", data_col="Region"),
-                ColumnParam(title="SSL", type="str", data_col="IsSSL"),
+                ColumnParam(
+                    title="SSL",
+                    type="str",
+                    data_col="IsSSL",
+                    normal=True,
+                    alarm=False,
+                ),
+                ColumnParam(
+                    title="Versioning",
+                    type="str",
+                    data_col="VersioningStatus",
+                    normal="Enabled",
+                ),
+                ColumnParam(
+                    title="LifeCycles",
+                    type="str",
+                    data_col="LifecycleRulesCount",
+                    alarm_lo=0,
+                ),
+                ColumnParam(
+                    title="Obj.Lock",
+                    type="str",
+                    data_col="ObjectLock",
+                    normal="Enabled",
+                ),
             ]
         )
+
         print()
         Table(params, self.buckets).print()
 
@@ -398,14 +581,23 @@ class S3(Base):
         print("")
         print(f"Bucket: {self.bucket}")
         print("")
-        print("Policies:")
+        print("---- Policies ->")
         pprint.pprint(self.bucket_info["Policies"])
+        print("-----------------")
+        print("")
+
+        print("---- Lifecyle Rules ->")
+        pprint.pprint(self.bucket_info["LifecycleRules"])
+        print("-----------------")
         print("")
 
         if self.bucket_info["IsSSL"]:
-            print(o7t.format_normal("SSL Request is enforced"))
+            print(o7t.format_normal("SSL Request: Enforced"))
         else:
-            print(o7t.format_alarm("SSL Request is not enforced"))
+            print(o7t.format_alarm("SSL Request: NOT Enforced"))
+        print(f"Versioning: {self.bucket_info['Versioning']['Status']}")
+        print(f"MFADelete: {self.bucket_info['Versioning']['MFADelete']}")
+        print("")
 
     # *************************************************
     #
@@ -486,9 +678,7 @@ class S3(Base):
             compact=True,
         )
         obj.add_option(
-            o7m.Option(
-                key="r", name="Raw", callback=lambda: pprint.pprint(self.objects)
-            )
+            o7m.Option(key="r", name="Raw", callback=lambda: pprint.pprint(self.objects))
         )
         obj.add_option(o7m.Option(key="u", name="Upload", callback=self.upload_file))
         obj.add_option(o7m.Option(key="int", name="Details", callback=self.menu_object))
@@ -517,7 +707,7 @@ class S3(Base):
         )
         obj.add_option(
             o7m.Option(
-                key="r", name="Raw", callback=lambda: pprint.pprint(self.objects)
+                key="r", name="Raw", callback=lambda: pprint.pprint(self.bucket_info)
             )
         )
         obj.add_option(o7m.Option(key="f", name="See Files", callback=self.menu_folder))
@@ -528,6 +718,20 @@ class S3(Base):
                 callback=self.set_ssl_requests_only,
             )
         )
+        obj.add_option(
+            o7m.Option(
+                key="add-lcr",
+                name="Add Basic Lifecycle Rule (delete non current after 1 year)",
+                callback=self.set_basic_lifecycle_fule,
+            )
+        )
+        obj.add_option(
+            o7m.Option(
+                key="m",
+                name="Get Bucket Metrics",
+                callback=self.get_bucket_metrics,
+            )
+        )
 
         obj.display_callback = self.display_bucket
         obj.loop()
@@ -553,6 +757,16 @@ class S3(Base):
                 callback=lambda: pprint.pprint(self.buckets),
             )
         )
+        obj.add_option(
+            o7m.Option(
+                key="l",
+                name="Load Details",
+                short="Load Details",
+                wait=False,
+                callback=self.load_buckets_extra,
+            )
+        )
+
         obj.add_option(
             o7m.Option(
                 key="int",
@@ -593,10 +807,6 @@ if __name__ == "__main__":
     # the_s3.display_folder()
 
     # the_s3.display_object()
+    the_s3.get_bucket_metrics(bucket="s3d-projets-archive")
 
     # theS3.MenuBuckets()
-
-    the_s3.load_bucket_extra("appstream2-36fb080bb8-ca-central-1-649732530399")
-    the_s3.load_bucket_extra(
-        "apistack-apibasicservicedownloadbucket31ba7d2a-jg2n5e641xqe"
-    )
