@@ -74,6 +74,7 @@ class SecurityHub(Base):
         self.df_standards_results: pd.DataFrame = None
         # self.df_standards_global : pd.DataFrame = None
         self.df_controls: pd.DataFrame = None
+        self.df_ctrl_def: pd.DataFrame = None
         self.df_controls_results: pd.DataFrame = None
         self.df_findings: pd.DataFrame = None
 
@@ -189,9 +190,7 @@ class SecurityHub(Base):
             standards.extend(page.get("StandardsSubscriptions", []))
 
         # self.df_standards = pd.DataFrame(standards)
-        logger.info(
-            f"load_enabled_standards: Number of standards found {len(standards)}"
-        )
+        logger.info(f"load_enabled_standards: Number of standards found {len(standards)}")
 
         df = pd.DataFrame(standards)
         df["Standards"] = df["StandardsArn"].str.split(":").str[-1]
@@ -218,6 +217,7 @@ class SecurityHub(Base):
 
         for standards, row in df_ready.iterrows():
             controls = []
+            definitions = []
 
             #
             # print(f'Loading controls for {standards} = {row["StandardsSubscriptionArn"]}')
@@ -227,7 +227,7 @@ class SecurityHub(Base):
             paginator = self.client.get_paginator("describe_standards_controls")
 
             for page in paginator.paginate(
-                StandardsSubscriptionArn=row["StandardsSubscriptionArn"]
+                StandardsSubscriptionArn=row["StandardsSubscriptionArn"],
             ):
                 controls.extend(page.get("Controls", []))
 
@@ -242,6 +242,25 @@ class SecurityHub(Base):
                     [self.df_controls, df_controls], ignore_index=True
                 )
 
+            # https://boto3.amazonaws.com/v1/documentation/api/1.26.88/reference/services/securityhub/client/list_security_control_definitions.html
+            paginator = self.client.get_paginator("list_security_control_definitions")
+
+            for page in paginator.paginate(
+                StandardsArn=row["StandardsArn"],
+            ):
+                definitions.extend(page.get("SecurityControlDefinitions", []))
+
+            # print(f'Number of controls found {len(controls)}')
+            df_definitions = pd.DataFrame(definitions)
+            df_definitions["Standards"] = standards
+
+            if self.df_ctrl_def is None:
+                self.df_ctrl_def = df_definitions
+            else:
+                self.df_ctrl_def = pd.concat(
+                    [self.df_ctrl_def, df_definitions], ignore_index=True
+                )
+
         self.df_controls["ControlStatusUpdatedAt"] = self.df_controls[
             "ControlStatusUpdatedAt"
         ].dt.tz_localize(None)
@@ -254,6 +273,16 @@ class SecurityHub(Base):
         self.df_standards["ControlsCount"] = gb["ControlId"].count()
         self.df_standards["ControlsDisabled"] = gb["IsDisabled"].sum()
 
+        self.df_ctrl_def["StandardsControl"] = self.df_ctrl_def["Standards"].str.replace(
+            "standards", "control"
+        )
+        self.df_ctrl_def["StandardsControl"] = (
+            self.df_ctrl_def["StandardsControl"]
+            + "/"
+            + self.df_ctrl_def["SecurityControlId"]
+        )
+        self.df_ctrl_def["ControlId"] = self.df_ctrl_def["SecurityControlId"]
+
     # *************************************************
     #
     # *************************************************
@@ -329,7 +358,7 @@ class SecurityHub(Base):
             lambda x: x.get("Text", "") if isinstance(x, dict) else ""
         )
         df["passed"] = (df["Status"] == "PASSED") | (
-            df["WorkflowStatus"] == "SUPPRESSED"
+            df["WorkflowStatus"].isin(["SUPPRESSED", "RESOLVED"])
         )
 
         df["SecurityControlId"] = df["Compliance"].apply(
@@ -351,8 +380,19 @@ class SecurityHub(Base):
         )
         df.rename(columns={"Name": "AccountName"}, inplace=True)
 
-        df["Standards"] = df["StandardsArn"].str.split(":").str[-1]
-        df["StandardsControl"] = df["StandardsControlArn"].str.split(":").str[-1]
+        # df["Standards"] = df["StandardsArn"].str.split(":").str[-1]
+        # Fanding can apply to multiple standards
+        df["Standards"] = df["Compliance"].apply(
+            lambda x: [
+                ass_std.get("StandardsId") for ass_std in x.get("AssociatedStandards", [])
+            ]
+            if isinstance(x, dict)
+            else []
+        )
+        df = df.explode("Standards")
+
+        df["StandardsControl"] = df["Standards"].str.replace("standards", "control")
+        df["StandardsControl"] = df["StandardsControl"] + "/" + df["SecurityControlId"]
 
         df["ResType"] = df["Resources"].apply(
             lambda x: x[0].get("Type", None) if isinstance(x, list) else None
@@ -360,6 +400,10 @@ class SecurityHub(Base):
 
         df["ResName"] = ""
 
+        # print(df[["Standards","StandardsControl"]])
+        # print(df.iloc[0])
+        # exit(0)
+
         # Find Ressource Name
         #  IF name is after last : in ID
         res_filter = df["ResType"].isin(
@@ -407,23 +451,21 @@ class SecurityHub(Base):
         )
 
         # IF name is in Tag Name
-        res_filter = df["ResType"].isin(
-            ["AwsEc2Vpc", "AwsEc2Subnet", "AwsEfsFileSystem"]
-        )
+        res_filter = df["ResType"].isin(["AwsEc2Vpc", "AwsEc2Subnet", "AwsEfsFileSystem"])
         df.loc[res_filter, "ResName"] = df[res_filter]["Resources"].apply(
             lambda x: x[0].get("Tags", {}).get("Name", "-")
         )
 
         # Verify for suppression flag
-        df["Suppression_Tag"] = df["ControlId"].apply(lambda x: f"sechub:suppress:{x}")
+        df["Suppression_Tag"] = df["SecurityControlId"].apply(
+            lambda x: f"sechub:suppress:{x}"
+        )
         df["Suppression_Reason"] = df.apply(
             lambda x: x["Resources"][0].get("Tags", {}).get(x["Suppression_Tag"], ""),
             axis=1,
         )
 
-        self.df_findings = df.sort_values(
-            by=["SeverityN", "Id"], ascending=[False, True]
-        )
+        self.df_findings = df.sort_values(by=["SeverityN", "Id"], ascending=[False, True])
         logger.info(
             f"df_findings: Number of standards found {len(self.df_findings.index)}"
         )
@@ -437,9 +479,7 @@ class SecurityHub(Base):
         matching_tables = []
 
         for table_name in dynamodb.list_tables()["TableNames"]:
-            table_arn = dynamodb.describe_table(TableName=table_name)["Table"][
-                "TableArn"
-            ]
+            table_arn = dynamodb.describe_table(TableName=table_name)["Table"]["TableArn"]
             response = dynamodb.list_tags_of_resource(ResourceArn=table_arn)
             tags = response["Tags"]
 
@@ -461,7 +501,7 @@ class SecurityHub(Base):
             print("Historical table not found")
             return
 
-        self.df_history = self.read_history_from_dynamodb(tables[0], number_days=90)
+        self.df_history = self.read_history_from_dynamodb(tables[0], number_days=180)
 
         return self
 
@@ -525,35 +565,46 @@ class SecurityHub(Base):
         df_std_ctrl["CheckFail"] = df_std_ctrl["CheckCount"] - df_std_ctrl["CheckPass"]
 
         # Create list of all Controls for all accounts
-        df_controls = self.df_controls[
+        df_controls = self.df_ctrl_def[
             [
                 "Standards",
-                "ControlStatus",
                 "StandardsControl",
                 "ControlId",
                 "SeverityRating",
             ]
         ].copy()
+        # Get the disabled controls flag
+        df_controls = df_controls.merge(
+            self.df_controls[["StandardsControl", "IsDisabled"]],
+            how="left",
+            on="StandardsControl",
+        )
+
         df_accounts["key"] = 1
         df_controls["key"] = 1
         df = pd.merge(left=df_accounts, right=df_controls, on="key").drop("key", axis=1)
         df.set_index(["Standards", "AwsAccountId", "StandardsControl"], inplace=True)
 
         # Merge with Findings results
-        df = df.merge(df_std_ctrl, how="left", left_index=True, right_index=True)
+        df = df.merge(df_std_ctrl, how="outer", left_index=True, right_index=True)
         df["CheckCount"] = df["CheckCount"].fillna(0)
         df["CheckPass"] = df["CheckPass"].fillna(0)
         df["CheckFail"] = df["CheckFail"].fillna(0)
+
+        # Fill with 0 if control is disabled
+        df["IsDisabled"] = df["IsDisabled"].astype(bool).fillna(False)
+        df.loc[df["IsDisabled"], "CheckCount"] = 0
+        df.loc[df["IsDisabled"], "CheckPass"] = 0
+        df.loc[df["IsDisabled"], "CheckFail"] = 0
+
         df["ControlNoData"] = df["CheckCount"] == 0
-        df["ControlActive"] = (df["CheckCount"] > 0) & (
-            df["ControlStatus"] == "ENABLED"
-        )
-        df["ControlPassed"] = (df["CheckPass"] == df["CheckCount"]) & df[
-            "ControlActive"
-        ]
+        df["ControlActive"] = df["CheckCount"] > 0
+
+        df["ControlPassed"] = (df["CheckPass"] == df["CheckCount"]) & df["ControlActive"]
         df["ComplianceStatus"] = "FAILED"
         df.loc[df["ControlPassed"], "ComplianceStatus"] = "PASSED"
         df.loc[df["ControlNoData"], "ComplianceStatus"] = "NO DATA"
+        df.loc[df["IsDisabled"], "ComplianceStatus"] = "DISABLED"
 
         self.df_controls_results = df
 
@@ -580,7 +631,7 @@ class SecurityHub(Base):
         df_std = pd.DataFrame(index=gb_std.groups.keys())
         df_std.index.names = ["Standards", "AwsAccountId"]
         df_std["AccountName"] = gb_std["AccountName"].first()
-        df_std["Controls"] = gb_std["ControlStatus"].count()
+        df_std["Controls"] = gb_std["ControlId"].count()
         df_std["ControlsActive"] = gb_std["ControlActive"].sum()
         df_std["ControlsPassed"] = gb_std["ControlPassed"].sum()
         df_std["ControlsFailed"] = df_std["ControlsActive"] - df_std["ControlsPassed"]
@@ -762,11 +813,11 @@ class SecurityHub(Base):
         standards = self.standard.name
 
         print("")
-        print(f'Name: {self.standard["Name"]}')
-        print(f'Description: {self.standard["Description"]}')
+        print(f"Name: {self.standard['Name']}")
+        print(f"Description: {self.standard['Description']}")
         print(f"Standards: {standards}")
         print("")
-        print(f'Status: {self.standard["StandardsStatus"]}')
+        print(f"Status: {self.standard['StandardsStatus']}")
         print("")
 
         try:
@@ -1050,7 +1101,7 @@ class SecurityHub(Base):
     def write_html_report_standards(self):
         """Save HTML report for standards"""
 
-        filename = f"aws-securityhub-report-standard-{datetime.datetime.now().isoformat()[0:19].replace(':','-')}.html"
+        filename = f"aws-securityhub-report-standard-{datetime.datetime.now().isoformat()[0:19].replace(':', '-')}.html"
         # filename= f"aws-securityhub-report-test.html"
         report = self.generate_html_report_standards(standards=self.standard.name)
 
@@ -1108,14 +1159,14 @@ class SecurityHub(Base):
         if self.df_history is None:
             self.load_historical_data()
 
-        filename = f"aws-securityhub-report-{standard['Short']}-{datetime.datetime.now().isoformat()[0:10].replace(':','-')}.pdf"
+        filename = f"aws-securityhub-report-{standard['Short']}-{datetime.datetime.now().isoformat()[0:10].replace(':', '-')}.pdf"
         filename = filename.replace(" ", "-")
         filename = os.path.join(folder, filename)
         standard_arn = standard["StandardsArn"].split(":")[-1]
 
         dfs = {
             "standards": self.df_standards,
-            "controls": self.df_controls,
+            "controls": self.df_ctrl_def,
             "findings": self.df_findings,
             "standards_results": self.df_standards_results,
             "controls_results": self.df_controls_results,
@@ -1135,7 +1186,7 @@ class SecurityHub(Base):
         """Display Security Hub"""
 
         # print(self.control)
-        print(f'Standard: {self.standard["Name"]}')
+        print(f"Standard: {self.standard['Name']}")
         # print(f'Id: {self.control["ControlId"]}')
         # print('')
         # print(f'Title: {self.control["Title"]}')
@@ -1290,7 +1341,7 @@ class SecurityHub(Base):
         df = self.df_menu_findings
         if self.account is not None:
             print(
-                f'Filtering by account: {self.account["AwsAccountId"]} - {self.account["AccountName"]}'
+                f"Filtering by account: {self.account['AwsAccountId']} - {self.account['AccountName']}"
             )
             df = df[df["AwsAccountId"] == self.account["AwsAccountId"]]
 
@@ -1337,15 +1388,15 @@ class SecurityHub(Base):
         """Display A Finding"""
 
         print("")
-        print(f'Source: {self.finding["ProductName"]}')
+        print(f"Source: {self.finding['ProductName']}")
         print(
-            f'Account / Region: {self.finding["AwsAccountId"]} / {self.finding["Region"]}'
+            f"Account / Region: {self.finding['AwsAccountId']} / {self.finding['Region']}"
         )
-        print(f'Type: {self.finding["Types"]}')
+        print(f"Type: {self.finding['Types']}")
         print()
-        print(f'Title: {self.finding["Title"]}')
+        print(f"Title: {self.finding['Title']}")
         print()
-        print(f'Description: {self.finding["Description"]}')
+        print(f"Description: {self.finding['Description']}")
         print()
         print("Severity")
         pprint.pprint(self.finding["Severity"])
@@ -1407,11 +1458,11 @@ class SecurityHub(Base):
             print(f"Number of accounts: {len(findings_compile.index)}")
             print(f"Total Findings : {total}")
             print("Security Hub Failed Findings")
-            print(f'   Critical: {findings_compile["Critical"].sum()}')
-            print(f'   High: {findings_compile["High"].sum()}')
-            print(f'   Medium: {findings_compile["Medium"].sum()}')
-            print(f'   Low: {findings_compile["Low"].sum()}')
-            print(f"   Total: {sh_failed} ({sh_failed/total*100:.1f}%)")
+            print(f"   Critical: {findings_compile['Critical'].sum()}")
+            print(f"   High: {findings_compile['High'].sum()}")
+            print(f"   Medium: {findings_compile['Medium'].sum()}")
+            print(f"   Low: {findings_compile['Low'].sum()}")
+            print(f"   Total: {sh_failed} ({sh_failed / total * 100:.1f}%)")
 
             print("")
             print("Top 10 Findings")
@@ -1420,9 +1471,7 @@ class SecurityHub(Base):
                     ColumnParam(title="Source", type="str", data_col="ProductName"),
                     ColumnParam(title="Account", type="str", data_col="AwsAccountId"),
                     ColumnParam(title="Region", type="str", data_col="Region"),
-                    ColumnParam(
-                        title="AccountName", type="str", data_col="AccountName"
-                    ),
+                    ColumnParam(title="AccountName", type="str", data_col="AccountName"),
                     ColumnParam(
                         title="Severity",
                         type="int",
@@ -1444,7 +1493,7 @@ class SecurityHub(Base):
     def to_excel(self):
         """Save to Excel"""
 
-        filename = f"aws-securityhub-{datetime.datetime.now().isoformat()[0:19].replace(':','-')}.xlsx"
+        filename = f"aws-securityhub-{datetime.datetime.now().isoformat()[0:19].replace(':', '-')}.xlsx"
         with pd.ExcelWriter(filename) as writer:
             df_parameters = pd.DataFrame(
                 [
@@ -1459,6 +1508,7 @@ class SecurityHub(Base):
 
             self.df_standards.to_excel(writer, sheet_name="Standards")
             self.df_controls.to_excel(writer, sheet_name="Controls")
+            self.df_ctrl_def.to_excel(writer, sheet_name="ControlsDef")
 
             for col in self.df_accounts:
                 if pd.api.types.is_datetime64tz_dtype(self.df_accounts[col]):
@@ -1470,9 +1520,7 @@ class SecurityHub(Base):
                 self.df_findings.to_excel(writer, sheet_name="Findings")
 
             if self.df_standards_results is not None:
-                self.df_standards_results.to_excel(
-                    writer, sheet_name="Standards_Results"
-                )
+                self.df_standards_results.to_excel(writer, sheet_name="Standards_Results")
                 self.df_controls_results.to_excel(writer, sheet_name="Controls_Results")
 
             if self.df_history is not None:
@@ -1500,7 +1548,7 @@ class SecurityHub(Base):
     def write_html_report(self):
         """Save HTML report"""
 
-        filename = f"aws-securityhub-report-{datetime.datetime.now().isoformat()[0:19].replace(':','-')}.html"
+        filename = f"aws-securityhub-report-{datetime.datetime.now().isoformat()[0:19].replace(':', '-')}.html"
         filename = "aws-securityhub-report-test.html"
         report = self.generate_html_report()
 
@@ -1538,16 +1586,12 @@ class SecurityHub(Base):
         df["IsMedium"] = (
             df["IsSecurityHub"] & (df["SeverityN"] >= 40) & (df["SeverityN"] < 70)
         )
-        df["IsLow"] = (
-            df["IsSecurityHub"] & (df["SeverityN"] < 40) & (df["SeverityN"] > 0)
-        )
+        df["IsLow"] = df["IsSecurityHub"] & (df["SeverityN"] < 40) & (df["SeverityN"] > 0)
 
         df["IsGuardDutyHigh"] = (
             df["IsGuardDuty"] & (df["SeverityL"] == "HIGH") & df["IsFailed"]
         )
-        df["IsGuardDutyLow"] = (
-            df["IsGuardDuty"] & ~df["IsGuardDutyHigh"] & df["IsFailed"]
-        )
+        df["IsGuardDutyLow"] = df["IsGuardDuty"] & ~df["IsGuardDutyHigh"] & df["IsFailed"]
 
         gb = df.groupby(["AwsAccountId"])
         df_ret = pd.DataFrame(index=gb.groups.keys())
@@ -1743,9 +1787,7 @@ class SecurityHub(Base):
                 section_params, section_findings[0:10].to_dict(orient="records")
             ).print()
             print("")
-            report.add_section(
-                title=f'{section["product"]} Findings', html=section_html
-            )
+            report.add_section(title=f"{section['product']} Findings", html=section_html)
 
         other_findings = findings[~findings["ProductName"].isin(all_product)]
         count = len(other_findings.index)
@@ -1753,9 +1795,7 @@ class SecurityHub(Base):
             section_params = TableParam(
                 columns=[
                     ColumnParam(title="Source", type="str", data_col="ProductName"),
-                    ColumnParam(
-                        title="Account Name", type="str", data_col="AccountName"
-                    ),
+                    ColumnParam(title="Account Name", type="str", data_col="AccountName"),
                     ColumnParam(title="Region", type="str", data_col="Region"),
                     ColumnParam(
                         title="Severity",
@@ -1780,9 +1820,7 @@ class SecurityHub(Base):
             section_html += Table(
                 section_params, other_findings[0:10].to_dict(orient="records")
             ).generate_html()
-            Table(
-                section_params, other_findings[0:10].to_dict(orient="records")
-            ).print()
+            Table(section_params, other_findings[0:10].to_dict(orient="records")).print()
             print("")
             report.add_section(title="Other Findings", html=section_html)
 
@@ -1849,9 +1887,7 @@ class SecurityHub(Base):
             repsonse = table.query(
                 KeyConditionExpression=(
                     boto3.dynamodb.conditions.Key("account").eq(account_id)
-                    & boto3.dynamodb.conditions.Key("date").between(
-                        last_day, current_day
-                    )
+                    & boto3.dynamodb.conditions.Key("date").between(last_day, current_day)
                 )
             )
             items.extend(repsonse["Items"])
@@ -1877,7 +1913,6 @@ class SecurityHub(Base):
 
         print(df.head(5))
         for _index, row in df.iterrows():
-
             short = row.get("short", None)
             today = row["date"]
             account_id = row["account"]
